#!ruby -Ks
# -*- mode: ruby; encoding: sjis -*-
# Last updated: <2014/01/25 22:20:46 +0900>

require 'dxruby'
require_relative 'globalwork'
require_relative 'gamemath'

# GC
class EnemyCannon < Sprite
  attr_accessor :px, :py, :imgs, :base_ang, :now_ang, :s_img
  attr_accessor :count, :c_dist, :cx, :cy
  attr_accessor :shot_time_def, :shot_timer, :shot_dist
  attr_accessor :dbg

  def initialize(x, y, imgs, udlr, shot_time=90, shot_dist=120, dbg=false)
    self.px = x
    self.py = y
    self.imgs = imgs
    self.image = imgs[0] # C̃x[X摜
    self.s_img = imgs[1] # C̖C摜
    set_xy
    super(self.x, self.y, self.image)

    # ĆA_ɂ
    self.center_x = self.image.width / 2
    self.center_y = self.image.height

    # C̉]ʒůWItZbg
    self.c_dist = 20

    # ㉺E̎w
    case udlr
    when "up" then self.angle = 0
    when "down" then self.angle = 180
    when "left" then self.angle = 270
    when "right" then self.angle = 90
    end

    # C̉͈͂߂邽߂̊pxݒ
    self.base_ang = (self.angle + 270) % 360
    self.now_ang = self.base_ang

    self.shot_time_def = shot_time
    self.shot_timer = rand(shot_time) + shot_time / 2
    self.shot_dist = shot_dist
    self.dbg = dbg
    self.count = 0

    # A^ݒ
    self.collision = [8, 2, 23, 31]
    self.collision_enable = true
  end

  # XV
  def update

    # BGXN[ɍ킹Ĉړ
    if self.dbg
      # mFp
      self.px -= $spd_x
      self.py -= $spd_y
    else
      # {
      self.px -= $g.bg.bg_x_spd
      self.py -= $g.bg.bg_y_spd
    end

    # vC[W擾
    if self.dbg
      # mFp
      plx, ply = Input.mousePosX, Input.mousePosY
    else
      # {
      plx, ply = $g.get_player_pos
    end

    # C̉]ʒu߂
    self.cx, self.cy = GameMath.get_vec(self.base_ang, self.c_dist, \
                                        self.px, self.py)

    # vC[ɌĖC̊pxݒ
    n = 24 # 360xɂ邩
    if false
      # ȂCꍇ
      if self.count % 10 == 0
        rad = GameMath.get_dir_rad(plx, ply, self.cx, self.cy)
        self.now_ang = GameMath.get_dir_rest(GameMath.rad2deg(rad), n)
      end
    else
      # Cꍇ
      if self.count % 8 == 0
        self.now_ang = GameMath.get_chase_angle(self.now_ang, (360.0 / n), \
                                              plx, ply, self.cx, self.cy)
        self.now_ang = GameMath.get_dir_rest(self.now_ang, n) % 360
      end
    end

    # C̊pxɐ
    self.now_ang = GameMath.get_dir_range(self.now_ang,
                                          self.base_ang, -90, 90)

    self.shot_timer -= 1
    if self.shot_timer <= 0
      # e
      unless GameMath.check_dist(plx, ply, self.cx, self.cy, self.shot_dist)
        # 苗OȂ̂ŒeĂ
        d = 26
        rad = GameMath.deg2rad(self.now_ang)
        x = self.cx + d * Math.cos(rad)
        y = self.cy + d * Math.sin(rad)
        if self.dbg
          # mFp
          $sprs.push(Shot.new(x, y, $imgs[2], self.now_ang, 2))
          self.shot_timer = self.shot_time_def
        else
          # {ԗp
          if $g.make_enemyshot(x, y, self.shot_dist, self.now_ang, 4)
            self.shot_timer = self.shot_time_def
          end
        end
      end
    end

    set_xy

    # ʊOɏo
    self.vanish if self.px < -64

    self.count += 1
  end

  def set_xy
    self.x = self.px - self.image.width / 2
    self.y = self.py - self.image.height
  end

  # `揈
  def draw
    # C`
    x = self.cx
    y = self.cy - self.s_img.height / 2
    Window.drawEx(x, y, self.s_img,
                  :angle=>(self.now_ang % 360),
                  :center_x=>0,
                  :center_y=>self.s_img.height / 2)

    # C`
    super
  end

  # vC[̒eɌĂ΂
  def hit(o)
    $g.add_score(10)
    ExplosionBorn.born_explosion(self.px, self.py, -2)
    self.collision_enable = false
    self.vanish
  end
end

if $0 == __FILE__
  # mFp

  # e
  class Shot < Sprite
    attr_accessor :px, :py, :dx, :dy
    def initialize(x, y, img, ang, spd)
      self.px = x
      self.py = y
      self.image = img
      set_xy
      super(self.x, self.y, self.image)
      rad = ang * Math::PI / 180.0
      self.dx = spd * Math.cos(rad)
      self.dy = spd * Math.sin(rad)
    end

    def update
      self.px += self.dx
      self.py += self.dy
      set_xy
      if self.px < 0 or self.px > Window.width or \
        self.py < 0 or self.py > Window.height
        self.vanish
      end
    end

    def set_xy
      self.x = self.px - self.image.width / 2
      self.y = self.py - self.image.height / 2
    end
  end

  $spd_x = 0.05
  $spd_y = 0

  $imgs = Image.loadTiles("res/cannon.png", 3, 1)
  $sprs = []
  x, y = 480, 240
  w = 150
  st = 20
  sd = 120
  $sprs.push(EnemyCannon.new(x, y + w, $imgs, "up", st, sd, true))
  $sprs.push(EnemyCannon.new(x, y - w, $imgs, "down", st, sd, true))
  $sprs.push(EnemyCannon.new(x + w, y, $imgs, "left", st, sd, true))
  $sprs.push(EnemyCannon.new(x - w, y, $imgs, "right", st, sd, true))
  # Window.fps = 15
  # Window.scale = 2
  Window.loop do
    break if Input.keyPush?(K_ESCAPE)
    Sprite.update($sprs)
    Sprite.draw($sprs)
  end
end
