#!ruby -Ks
# -*- mode: ruby; encoding: sjis -*-
# Last updated: <2014/01/25 03:05:41 +0900>
#
# Q[֌W̌vZp\bh
#
# ruby gamemath.rb 1 ƂwŁAeXgނς

require 'dxruby'

# vZp\bh
class GameMath

  class << self

    #
    # x烉WAւ̕ϊ
    #
    # @param [float] deg pxBPʂ͓x
    # @retrun [float] pxBPʂ̓WA
    #
    def deg2rad(deg)
      return (deg * Math::PI / 180.0)
    end

    #
    # WAxւ̕ϊ
    #
    # @param [float] rad pxBPʂ̓WA
    # @retrun [float] pxBPʂ͓x
    #
    def rad2deg(rad)
      return (rad * 180.0 / Math::PI)
    end

    #
    # WڕWWւ̊pxԂ
    #
    # @param [int] tgt_x ڕWWx
    # @param [int] tgt_y ڕWWy
    # @param [int] base_x Wx
    # @param [int] base_y Wy
    # @return [float] ڕWW܂ł̊pxBPʂ̓WA
    #
    def get_dir_rad(tgt_x, tgt_y, base_x, base_y)
      tx = tgt_x - base_x
      ty = tgt_y - base_y
      return Math.atan2(ty, tx)
    end

    #
    # pxnɐĕԂ
    #
    # @param [float] ang ̊px
    # @param [float] rest ɂ邩BftHg16
    # @return [float] restɐpx
    #
    def get_dir_rest(ang, rest=16)
      d = 360.0 / rest
      dir = ((ang + (d / 2)) % 360.0).div(d)
      return dir * d
    end

    #
    # pxƑx()x,ÿړʂ߂(x^)
    #
    # @param [float] ang pxBPʂ͓x
    # @param [float] spd x()
    # @param [float] bx  _ƂȂWxBȗ0
    # @param [float] by  _ƂȂWyBȗ0
    # @return [Array] ړx,y
    #
    def get_vec(ang, spd, bx=0, by=0)
      return GameMath.get_vec_rad(GameMath.deg2rad(ang), spd, bx, by)
    end

    #
    # pxƑx()x,ÿړʂ߂(WA^)
    #
    # @param [float] ang pxBPʂ̓WA
    # @param [float] spd x()
    # @param [float] bx  _ƂȂWxBȗ0
    # @param [float] by  _ƂȂWyBȗ0
    # @return [Array] ړx,y
    #
    def get_vec_rad(rad, spd, bx=0, by=0)
      x = spd * Math.cos(rad) + bx
      y = spd * Math.sin(rad) + by
      return x, y
    end

    #
    # W]
    #
    # @param [float] x wWx
    # @param [float] y wWy
    # @param [float] rad ]pxBPʂ̓WA
    # @return [Array] ]{ItZbglZx,yW
    #
    def get_rot_pos(x, y, rad)
      nx = x * Math.cos(rad) - y * Math.sin(rad)
      ny = x * Math.sin(rad) + y * Math.cos(rad)
      return nx, ny
    end

    #
    # OςԂ
    #
    # @param [int] tgt_x ڕWxNgx
    # @param [int] tgt_y ڕWxNgy
    # @param [int] base_x xNgx
    # @param [int] base_y xNgy
    # @return [float] OρB
    #                 O > 0 ȂAڕW͉EɂA
    #                 O < 0 ȂAڕW͍ɂ
    #                 O = 0 ȂA
    #
    def get_cross_product(tgt_x, tgt_y, base_x, base_y)
      return base_x * tgt_y - base_y * tgt_x
    end

    #
    # XɖڕWpx߂
    #
    # @param [float] ang ݂̊pxBPʂ͓x
    # @param [float] ang_spd pxωʁBPʂ͓x
    # @param [float] tgt_x ڕWWx
    # @param [float] tgt_y ڕWWy
    # @param [float] base_x Wx
    # @param [float] base_y Wy
    # @return [float] VpxBPʂ͓x
    #
    def get_chase_angle(ang, ang_spd, tgt_x, tgt_y, base_x, base_y)
      ang %= 360
      tgt_ang = GameMath.rad2deg(GameMath.get_dir_rad(tgt_x, tgt_y, base_x, base_y))
      tgt_ang %= 360
      d = tgt_ang - ang
      d += 360 if d < -180
      d -= 360 if d > 180
      if d > ang_spd
        ang += ang_spd
      elsif d < -ang_spd
        ang -= ang_spd
      else
        ang = tgt_ang
      end
      return ang
    end

    #
    # w͈͂Ɏ܂pxԂ
    #
    # @param [float] now_ang ݂̊pxBPʂ͓x
    # @param [float] base_ang pxBPʂ͓x
    # @param [float] min_d px͈͂̍ŏlZopBpxɉZl
    # @param [float] max_d px͈͂̍őlZopBpxɉZl
    # @return [float] ͈͂Ɏ܂pxBPʂ͓x
    #
    def get_dir_range(now_ang, base_ang, min_d, max_d)
      base_ang %= 360
      now_ang %= 360

      # ݊pxApx̐΂̊px𒴂px␳
      if now_ang > base_ang + 180
        now_ang -= 360
      elsif now_ang <= base_ang - 180
        now_ang += 360
      end

      min_ang = base_ang + min_d
      max_ang = base_ang + max_d
      now_ang = min_ang if now_ang < min_ang
      now_ang = max_ang if now_ang > max_ang
      return now_ang
    end

    #
    # 2_̍WԂ̋w苗菬Ԃ
    #
    # @param [int] tx ڕWWx
    # @param [int] ty ڕWWy
    # @param [int] bx Wx
    # @param [int] by Wy
    # @param [int] dist 苗
    # @return {Boolean] trueȂA苗ɓĂBfalseȂA苗O
    #
    def check_dist(tx, ty, bx, by, dist)
      w = tx - bx
      h = ty - by
      return (w * w + h * h <= dist * dist)? true : false
    end

  end
end

if $0 == __FILE__
  # mF

  # ނς
  if ARGV.length >= 1
    test_kind = ARGV[0].to_i
  else
    test_kind = 0
  end

  font = Font.new(16)
  fh = 20

  bx = 320
  by = 240
  ang = 0
  base_ang = 0

  Window.loop do
    break if Input.keyPush?(K_ESCAPE)
    px, py = Input.mousePosX, Input.mousePosY

    case test_kind
    when 0
      # (px)ɐeXg
      # nɐ

      n = 12
      rad = GameMath.get_dir_rad(px, py, bx, by) # {̊px
      new_deg = GameMath.get_dir_rest(GameMath.rad2deg(rad), n) # px

      # `
      d = 360 / n
      s = d / 2
      n.times do |i|
        nx, ny = GameMath.get_vec(s + d * i, 80, bx, by)
        Window.drawLine(bx, by, nx, ny, C_RED)
      end

      nx, ny = GameMath.get_vec(GameMath.rad2deg(rad), 200, bx, by)
      Window.drawLine(bx, by, nx, ny, C_CYAN)

      rad = GameMath.deg2rad(new_deg)
      nx, ny = GameMath.get_vec(GameMath.rad2deg(rad), 150, bx, by)
      Window.drawLine(bx, by, nx, ny, C_YELLOW)

      Window.drawFont(0, 0, "#{n}ɂ鐧", font)

    when 1
      # OςgĖڕWǔ悤ɊpxςeXg
      # vvkĂ܂

      ang_spd = 3

      # ڕWxNg
      tx = px - bx
      ty = py - by

      # xNg
      vx = Math.cos(GameMath.deg2rad(ang))
      vy = Math.sin(GameMath.deg2rad(ang))

      # Oς߂
      cp = GameMath.get_cross_product(tx, ty, vx, vy)

      # Oς̕āAڕWEɂ邩ɂ邩
      ang += ang_spd if cp > 0
      ang -= ang_spd if cp < 0
      ang %= 360

      # `
      nx, ny = GameMath.get_vec(ang, 200, bx, by)
      Window.drawLine(bx, by, nx, ny, C_CYAN)

      Window.drawFont(0, 0, "Oς̕Œǔ", font)

    when 2
      # ڕWւ̊px߂āAǔ悤ɊpxςeXg

      ang = GameMath.get_chase_angle(ang, 2, px, py, bx, by)

      # `
      nx, ny = GameMath.get_vec(ang, 200, bx, by)
      Window.drawLine(bx, by, nx, ny, C_CYAN)

      Window.drawFont(0, 0, "pxĒǔ", font)

    when 3
      # pxɐ

      base_ang += Input.x * 2 # pxEL[ŕύX

      rad = GameMath.get_dir_rad(px, py, bx, by) # {̊px
      ang = GameMath.rad2deg(rad)

      # pxvX}CiX aw ͈̔͂Ɋpx𐧌
      aw = 90
      ang = GameMath.get_dir_range(ang, base_ang, -aw, aw)

      # `
      nx, ny = GameMath.get_vec(base_ang, 250, bx, by)
      Window.drawLine(bx, by, nx, ny, C_RED)
      nx, ny = GameMath.get_vec(base_ang - aw, 150, bx, by)
      Window.drawLine(bx, by, nx, ny, C_YELLOW)
      nx, ny = GameMath.get_vec(base_ang + aw, 150, bx, by)
      Window.drawLine(bx, by, nx, ny, C_YELLOW)

      nx, ny = GameMath.get_vec(ang, 200, bx, by)
      Window.drawLine(bx, by, nx, ny, C_CYAN)

      Window.drawFont(0, 0, "pxɐ : EL[ŊpxύX", font)
    end
  end
end
