#!ruby -Ks
# -*- mode: ruby; encoding: sjis -*-
# Last updated: <2014/01/27 08:43:35 +0900>

require 'dxruby'

#
# ShadergMETALBLACK̃^CgʂۂNX
#
class MbRaster

  @@hlsl = <<EOS
  float ang;
  float lvl;
  float spd;
  float alpha;
  texture tex0;

  sampler Samp = sampler_state
  {
    Texture =<tex0>;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    MipFilter = LINEAR;
    AddressU = WRAP;
    AddressV = WRAP;
    // AddressV = BORDER;
  };

  struct PixelIn
  {
    float2 UV : TEXCOORD0;
  };
  struct PixelOut
  {
    float4 Color : COLOR0;
  };

  PixelOut PS1(PixelIn input)
  {
    PixelOut output;
    // input.UV.x = input.UV.x + sin(radians(input.UV.x * 360 * spd + ang)) * lvl * 0.25;
    input.UV.y = input.UV.y + sin(radians(input.UV.y * 360 * spd - ang)) * lvl;
    output.Color = tex2D( Samp, input.UV );
    output.Color.a *= alpha;
    return output;
  }

  technique MbRasScroll
  {
   pass P0
   {
    PixelShader = compile ps_2_0 PS1();
   }
  }
EOS

  attr_accessor :core, :shader, :image, :w, :h, :tilew, :tileh
  attr_accessor :rt, :rt_w, :rt_h, :rt_scaley

  #
  # 
  #
  # @param [Image] image gp摜
  # @param [int] w `扡BnilȂEChE
  # @param [int] h `cBnilȂEChEc
  #
  def initialize(image, w=nil, h=nil)
    self.core = Shader::Core.new(@@hlsl,
                                 {:ang=>:float, :lvl=>:float,
                                   :spd=>:float, :alpha=>:float})
    self.shader = Shader.new(self.core, "MbRasScroll")

    @image = image
    @w = (w == nil)? Window.width : w
    @h = (h == nil)? Window.height : h

    # ʂ𕢂B^C߂
    @tilew = @w.quo(@image.width).ceil
    @tileh = @h.quo(@image.height).ceil

    # RenderTarget ̕KvTCY߂
    @rt_w = @image.width * @tilew
    @rt_h = @image.height * @tileh

    # yŊgkĕ`cɍ킹
    @rt_scaley = @h.quo(@rt_h)

    # Rendertarget
    @rt = RenderTarget.new(@rt_w, @rt_h)
  end

  #
  # `揈
  #
  # @param [int] x `xW
  # @param [int] y `yW
  # @param [float] u XN[ʒu
  # @param [float] v cXN[ʒu
  # @param [float] ang sin̊JnpxBPʂ͓x
  # @param [float] lvl sin̐U
  # @param [float] spd sin̊pxω
  # @param [Hash] opts IvV
  # @option opts [int] z `扜s
  # @option opts [float] alpha xB0.0œB1.0ŕs
  # @option opts [Object] blend @B:add  :alpha g͂
  #
  def draw(x, y, u, v, ang, lvl, spd, opts={})
    alpha = (opts.has_key?(:alpha))? opts[:alpha] : 1.0
    blend = (opts.has_key?(:blend))? opts[:blend] : :alpha
    z = (opts.has_key?(:z))? opts[:z] : 0

    self.shader.ang = ang
    self.shader.lvl = lvl
    self.shader.spd = spd
    self.shader.alpha = alpha

    # Shader QƂ摜 RenderTarget ɍ쐬
    @rt.draw_tile(0, 0, [[0]], [@image], u, v, @tilew, @tileh).update

    # Shader gĕ`
    Window.drawEx(x, y, @rt, :shader=>self.shader,
                  :scaley=>@rt_scaley, :z=>z, :blend=>blend)
  end

  #
  # mFpBObh摜𐶐ĕԂ
  #
  # @param [int] w 摜
  # @param [int] h 摜c
  # @return [Image] 摜
  #
  def self.get_grid_image(w, h)
    image = Image.new(w, h, [0, 255, 0])
    image.box_fill(0, 0, w / 2 - 1, h / 2 - 1, [150,250,150])
    image.box_fill(w / 2, 0, w - 1, h / 2 - 1, [100,250,100])
    image.box_fill(0, h / 2, w / 2 - 1, h - 1, [200,250,200])
    image.box_fill(w / 2, h / 2, w - 1, h - 1, [0,220,0])
    return image
  end
end

# ----------------------------------------
# ȉ͎gp

if __FILE__ == $0

  image = Image.load("res/titlelogo.png")
  bg = MbRaster.new(image)

  u = 0
  v = 0
  ang = 360
  lvldef = 0.35
  lvl = lvldef
  spd = 1.0

  cnt = 0

  font = Font.new(14)

  Window.loop do
    break if Input.keyPush?(K_ESCAPE)

    # }EXJ[\WŃp[^ςĂ݂
    ang = Input.mousePosX * 360 / 640.0
    lvl = Input.mousePosY / 240.0  - 1.0
    # spd = Input.mousePosY / 480.0

    bg.draw(0, 0, u, v, ang, lvl, spd)
    # u += 1
    # v += 2

    Window.drawFont(4, 4, "ang=#{ang} , lvl=#{lvl} , spd=#{spd}", font)
    cnt += 1
  end
end
