#!ruby -Ks
# -*- mode: ruby; encoding: sjis -*-
# Last updated: <2014/01/27 01:06:05 +0900>

require 'dxruby'
require_relative 'gamemath'

# Stage1̃{XG
class Stage1Boss < Sprite

  # r
  class Arm < Sprite
    attr_accessor :vflip

    def initialize(x, y, img, vflip, z)
      self.image = img
      self.vflip = vflip
      self.z = z
      set_xy(x, y, 0)
      super(self.x, self.y, self.image)
    end

    # \WݒBeobjĂłčXV
    def set_xy(x, y, ang)
      yd = (self.vflip)? -1 : 1
      self.center_x = 25
      self.center_y = 60
      self.x = x + 12 - self.center_x
      self.y = y - 114 * yd - self.center_y
      self.scale_x = 1
      self.scale_y = yd
      self.angle = ang * yd
    end
  end

  # {fB
  class BodySpr < Sprite
    attr_accessor :flash_value

    def initialize(x, y, img, z)
      set_xy(x, y)
      self.image = img
      self.z = z
      self.flash_value = 0
      super(self.x, self.y, self.image)

      # A^͈́Bž`œnƁÃA^͈͂ݒł
      self.collision = [
        [76, 16, 117, 31],
        [56, 31, 204, 57],
        [41, 56, 204, 95],
        [41, 161, 204, 201],
        [56, 200, 204, 226],
        [76, 224, 117, 240]
      ]
      self.collision_enable = true
    end

    # \WݒBeobjĂłčXV
    def set_xy(x, y)
      self.x = x - 168
      self.y = y - 128
    end

    def update
      if self.flash_value > 0
        # tbVp̒l炷
        self.flash_value -= 32
        self.flash_value = 0 if self.flash_value <= 0
      end
    end

    def draw
      super

      # tbVp̏
      if self.flash_value > 0
        Window.drawEx(self.x, self.y, self.image, :z=>self.z,
                      :alpha=>self.flash_value,
                      :blend=>:add)
      end
    end

    # tbVL
    def set_flash
      self.flash_value = 255 if self.flash_value <= 0
    end
  end

  # {XǦe1
  class BossShot1 < Sprite
    attr_accessor :px, :py, :spd

    def initialize(x, y, img, spd, z)
      self.px = x
      self.py = y
      self.image = img
      set_xy(self.px, self.py)
      super(self.x , self.y, self.image)
      self.z = z
      self.spd = spd
      self.collision = [9, 1, 73, 9]
      self.collision_enable = true
    end

    def update
      self.px += self.spd
      set_xy(self.px, self.py)
      if self.x + self.image.width < 0
        self.vanish
      end
    end

    def set_xy(x, y)
      self.x = x
      self.y = y - self.image.height / 2
    end
  end

  # {XǦ[U[
  class BossLaser < Sprite
    attr_accessor :px, :py
    attr_accessor :guide_img, :laser_img
    attr_accessor :timer, :count, :step
    attr_accessor :ang

    def initialize(x, y, img, guide_img, ang, vflip, z)
      self.px = x
      self.py = y
      self.laser_img = img
      self.guide_img = guide_img
      self.image = guide_img
      self.center_x = self.image.width
      self.center_y = 7
      set_xy(self.px, self.py)
      super(self.x , self.y, self.image)
      self.ang = ang
      self.angle = ang
      self.z = z
      self.alpha = 255
      self.blend = :add
      self.scale_x = 1.0
      self.scale_y = (vflip)? -1.0 : 1.0
      self.count = 0
      self.step = 0
      self.timer = 45
      self.collision = [0, 2, 64, 12]
      self.collision_enable = false
      self.collision_sync = true
    end

    def update
      case self.step
      when 0
        # [U[̌ꏊu\
        self.scale_x = rand(1300).quo(100)
        self.alpha = rand(64) + 192
        self.timer -= 1
        if self.timer <= 0
          self.step += 1
          self.image = self.laser_img
          self.center_x = self.image.width
          self.center_y = 7
          self.scale_x = 13.0
          self.scale_y = 1.0
          self.angle = self.ang
          self.alpha = 255
          self.timer = 15
          self.collision_enable = true
        end

      when 1
        # [U[˒
        self.scale_y = (self.count % 3) * 0.3 + 1.0
        self.timer -= 1
        if self.timer <= 0
          self.step += 1
          self.collision_enable = false
        end

      when 2
        # [U[Œ
        self.alpha -= 16
        if self.alpha <= 0
          self.alpha = 0
          self.vanish
        end
      end

      self.count += 1
    end

    def set_xy(x, y)
      self.x = x - self.center_x
      self.y = y - self.center_y
    end
  end

  STEP_BORN_DEMO = 0
  STEP_MOVE = 1
  STEP_SHOT = 2
  STEP_LASER = 3
  STEP_WAIT = 4
  STEP_DEAD_DEMO = 5

  DAMAGE_DEF = 120
  TIMER_DEF = 92
  ATACK_PAT = ["shot", "laser"]
  # ATACK_PAT = ["laser"]

  attr_accessor :boss_imgs
  attr_accessor :px, :py, :imgs
  attr_accessor :old_px, :old_py
  attr_accessor :arm, :arm_ang, :arm_ang_spd
  attr_accessor :body
  attr_accessor :count, :timer, :step, :atack_kind
  attr_accessor :kind, :damage
  attr_accessor :dy
  attr_accessor :dbg

  def initialize(imgs, dbg = false)
    self.dbg = dbg

    x = Window.width + 169
    y = Window.height / 2
    self.kind = 0
    self.damage = DAMAGE_DEF

    self.px = x
    self.py = y
    self.old_px = x
    self.old_py = y
    self.boss_imgs = imgs
    self.imgs = imgs["core"].sliceTiles(3, 1)
    self.image = self.imgs[self.kind]
    self.z = 0
    set_xy(self.px, self.py)
    super(self.x, self.y, self.image)

    self.body = BodySpr.new(x, y, imgs["body"], -2)

    self.arm = []
    self.arm_ang = 0
    self.arm_ang_spd = -0.6
    self.arm.push(Arm.new(x, y, imgs["arm"], false, -3))
    self.arm.push(Arm.new(x, y, imgs["arm"], true, -3))
    self.arm_ang = 0

    self.count = 0
    self.atack_kind = 0
    self.step = 0

    if self.dbg
      # mFp
      $sprs.push(self.arm[0])
      $sprs.push(self.arm[1])
      $sprs.push(self.body)
    else
      # {ԗp
      $g.sprs_enemy.push(self.arm[0])
      $g.sprs_enemy.push(self.arm[1])
      $g.sprs_enemy.push(self.body)
    end

    self.collision = [ 0, 0, 16, 16]
    self.collision_enable = true
  end

  def update
    case self.step
    when STEP_BORN_DEMO
      # o

      unless self.dbg
        # _CBG̃XN[xω
        $g.bg.add_ras_bg_spd(0.05, 0.02)
      end

      self.px -= 2
      if self.px <= 550
        self.dy = -3.5
        self.timer = TIMER_DEF
        self.step = STEP_MOVE
      end

    when STEP_MOVE
      # ړ

      # r̊pxύX
      self.arm_ang += self.arm_ang_spd
      self.arm_ang_spd *= -1 if self.arm_ang <= -40 or self.arm_ang >= 0

      # ㉺Ɉړ
      self.py += self.dy
      cy = Window.height / 2
      yh = 90
      self.dy *= -1 if self.py  < cy - yh or self.py > cy + yh

      # 莞Ԃ߂UXebvɈڍs
      self.timer -= 1
      if self.timer <= 0
        case ATACK_PAT[self.atack_kind]
        when "shot" then self.step = STEP_SHOT
        when "laser" then self.step = STEP_LASER
        end
        self.atack_kind = (self.atack_kind + 1) % ATACK_PAT.length
        self.timer = 60
      end

    when STEP_SHOT
      # ʏe
      x = self.px - 150
      yh = 51
      spd = -8
      spr0 = BossShot1.new(x, self.py - yh, self.boss_imgs["shot1"], spd, -4)
      spr1 = BossShot1.new(x, self.py + yh, self.boss_imgs["shot1"], spd, -4)

      if self.dbg
        # mFp
        $sprs.push(spr0)
        $sprs.push(spr1)
      else
        # {ԗp
        $g.sprs_enemyshot.push(spr0)
        $g.sprs_enemyshot.push(spr1)
      end
      self.timer = 90
      self.step = STEP_WAIT

    when STEP_LASER
      # [U[
      born_laser(self.px, self.py, self.arm_ang, false, 0)
      born_laser(self.px, self.py, self.arm_ang, false, 1)
      born_laser(self.px, self.py, self.arm_ang, true, 0)
      born_laser(self.px, self.py, self.arm_ang, true, 1)
      self.timer = 120
      self.step = STEP_WAIT

    when STEP_WAIT
      # ̈ړ܂ł̑҂
      self.timer -= 1
      if self.timer <= 0
        self.step = STEP_MOVE
        self.timer = TIMER_DEF
      end

    when STEP_DEAD_DEMO
      # SÕf

      # r𐦂ŐU
      self.arm_ang += self.arm_ang_spd * 4
      self.arm_ang_spd *= -1 if self.arm_ang <= -45 or self.arm_ang >= 45

      # \ʒuK^K^ƕς
      w = 32
      self.px = self.old_px + rand(w) - (w / 2)
      self.py = self.old_py + rand(w) - (w / 2)

      unless self.dbg
        # {ԗp
        if self.count % 3 == 0
          # Aj\
          w = 300
          x = self.px + rand(w) - (w / 2)
          y = self.py + rand(w) - (w / 2)
          ExplosionBorn.born_explosion(x, y, 3, false)
        end

        if self.count % 10 == 0
          # O\
          w = 150
          x = self.px + rand(w) - (w / 2)
          y = self.py + rand(w) - (w / 2)
          ExplosionBorn.born_ring(x, y, 3)
        end
      end

      self.timer -= 1
      if self.timer <= 0
        self.arm[0].vanish
        self.arm[1].vanish
        self.body.vanish
        self.vanish

        unless self.dbg
          $g.set_flash() # ʂtbV

          # 唚GtFNg𔭐
          spr = ExplosionBorn2.new(self.px, self.py, $g.explo2_imgs)
          $g.sprs_explosion.push(spr)
        end
      end

    end

    # Wݒ
    set_xy(self.px, self.py)

    # r̕\ʒuύX
    self.arm[0].set_xy(self.px, self.py, self.arm_ang)
    self.arm[1].set_xy(self.px, self.py, self.arm_ang)

    # ̂̕\ʒuύX
    self.body.set_xy(self.px, self.py)

    self.count += 1
  end

  # \Wݒ
  def set_xy(x, y)
    self.x = x - 8
    self.y = y - 8
  end

  # [U[{
  def born_laser(bx, by, ang, vflip, kind)
    yd = (vflip)? -1 : 1
    bx += 12
    by -= (114 * yd)
    x = -24
    y = ((kind == 0)? -28 : -51)  * yd
    ang *= yd

    nx, ny = GameMath.get_rot_pos(x, y, GameMath.deg2rad(ang))
    spr = BossLaser.new(nx + bx, ny + by,
                        self.boss_imgs["laser"],
                        self.boss_imgs["guide"],
                        ang, vflip, -4)
    if self.dbg
      # mFp
      $sprs.push(spr)
    else
      $g.sprs_enemyshot.push(spr)
    end

  end

  # vC[̒eĂ΂
  def hit(o)
    return if self.step == STEP_BORN_DEMO or self.step >= STEP_DEAD_DEMO

    $g.add_score(5) unless self.dbg

    self.body.set_flash

    self.damage -= 1
    if self.damage <= 0
      # HPȂ
      set_dead
    elsif self.damage < DAMAGE_DEF / 3
      # HP1/3菭Ȃ
      self.kind = 2
      self.image = self.imgs[self.kind]
    elsif self.damage < DAMAGE_DEF * 2 / 3
      # HP2/3菭Ȃ
      self.kind = 1
      self.image = self.imgs[self.kind]
    end
  end

  # SfJnݒ
  def set_dead
    self.step = STEP_DEAD_DEMO
    self.timer = 210
    self.old_px = self.px
    self.old_py = self.py
    self.count = 0
    self.collision_enable = false

    unless self.dbg
      $g.set_flash # ʂtbV
      $g.clean_all_enemy_shot # GeSď
    end
  end

  #
  # {XGp̉摜ǂݍ
  #
  # @return [Array] ImagezԂ
  #
  def self.load_images
    imgs = {}
    [
      ["res/boss_arm3.png", "arm"],
      ["res/boss_body.png", "body"],
      ["res/boss_core.png", "core"],
      ["res/boss_shot1.png", "shot1"],
      ["res/boss_laser.png", "laser"],
      ["res/boss_laser_guide.png", "guide"],
    ].each do |d|
      fn, name = d
      imgs[name] = Image.load(fn)
    end
    return imgs
  end
end

if $0 == __FILE__
  # mF

  # ̃vC[L
  class Player < Sprite
    def initialize
      self.image = Image.new(2, 2, C_WHITE)
      self.collision = [1, 1]
      self.collision_enable = true
      super(0, 0, self.image)
    end

    def update
      self.x = Input.mousePosX
      self.y = Input.mousePosY
    end

    def hit(o)
      Window.bgcolor = C_BLUE
    end
  end

  # 摜ǂݍ
  imgs = Stage1Boss.load_images

  # XvCg
  $sprs = []
  $sprs.push(Stage1Boss.new(imgs, true))

  $sprs_player = []
  $sprs_player.push(Player.new)

  pause_fg = false
  next_fg = false

  Window.loop do
    Window.bgcolor = C_BLACK
    break if Input.keyPush?(K_ESCAPE)

    if pause_fg
      pause_fg = false if Input.keyPush?(K_P)
      next_fg = (Input.keyPush?(K_N))? true : false
    else
      next_fg = true
      pause_fg = true if Input.keyPush?(K_P)
    end

    if next_fg
      Sprite.update($sprs)
      Sprite.update($sprs_player)

      Sprite.check($sprs_player, $sprs)
      Sprite.check($sprs, $sprs_player)
    end

    Sprite.draw($sprs)
    Sprite.draw($sprs_player)
  end
end
