#!ruby -Ks
# -*- mode: ruby; encoding: sjis -*-
# Last updated: <2014/01/13 13:55:40 +0900>

require 'dxruby'

# ShadergăX^XN[NX
#
# FԂۂBG摜X^[XN[
#
class StarRasterScrollShader < DXRuby::Shader
  hlsl = <<EOS
  float2 size;    // 摜TCY
  float d;        // XN[ʒu
  float spd;      // X^[̕ω
  float afactor;  // At@l
  texture tex0;

  // X^[̃XN[ʁB16X^[JԂ
  float dd[16] = {0, 8, 4, 12, 2, 10, 6, 14, 1, 9, 5, 13, 3, 11, 7, 15};

  sampler Samp0 = sampler_state
  {
    Texture =<tex0>;

    // Ɖʃ``
    // MinFilter = LINEAR;
    // MagFilter = LINEAR;
    // MipFilter = LINEAR;

    // Ȃ``Ȃ
    MinFilter = POINT;
    MagFilter = POINT;
    MipFilter = NONE;

    AddressU = WRAP;
    AddressV = WRAP;
  };

  struct PixelIn
  {
    float2 UV : TEXCOORD0;
  };

  struct PixelOut
  {
    float4 Color : COLOR0;
  };

  PixelOut PS(PixelIn input)
  {
    PixelOut output;
    input.UV.x += ((d + (d * spd * dd[fmod(input.UV.y * size.y, 16)])) / size.x);
    output.Color = tex2D( Samp0, input.UV );
    output.Color.a *= afactor;
    return output;
  }

  technique StarRasterScroll
  {
    pass P0
    {
      PixelShader = compile ps_2_0 PS();
    }
  }
EOS

  @@core = DXRuby::Shader::Core.new(hlsl,
                                    {
                                      :size => :float,
                                      :d => :float,
                                      :spd => :float,
                                      :afactor => :float
                                    })

  attr_accessor :speed

  #
  # 
  #
  # @param [float] speed   XN[x
  # @param [float] v       X^[̑xωʁB傫Ƒx傫Ȃ
  # @param [int]   width   ʉBȗƃEChE
  # @param [int]   height  ʏcBȗƃEChEc
  #
  def initialize(speed=1, v=0.4, width=nil, height=nil)
    super(@@core, "StarRasterScroll")
    w, h = Window.width, Window.height
    w = width if width
    h = height if height
    self.size = [w, h]
    self.d = 0.0
    self.spd = v
    self.afactor = 1.0
    @speed = speed
  end

  #
  # XN[lXV
  #
  def update
    self.d += @speed
  end

  #
  # XN[l
  #
  def reset_scroll
    self.d = 0
  end

  #
  # XN[xύX
  #
  # @param [float] speed   XN[x
  # @param [float] v       X^[̑xωʁB傫Ƒx傫Ȃ
  #
  def set_speed(speed=1, v=0.4)
    @speed = speed
    self.spd = v
  end

  #
  # At@lύX
  #
  # @param [float] v   At@lA0.0œA1.0ŕs
  #
  def set_alpha(v)
    self.afactor = v
  end

  #
  # `ꂽwip摜쐬ĕԂ
  #
  # @param [int] width      摜(ȗWindow)
  # @param [int] height     摜c(ȗWindowc)
  # @param [int] num        Uzhbg
  # @param [bool] len_sync  trueȂ̒X^[XN[xɍ킹
  #                         ftHgfalse
  #
  # @return [Image] 쐬摜(Image)
  #
  def get_star_image(width=nil, height=nil, num=0x3fff, len_sync=false)
    width = Window.width unless width
    height = Window.height unless height
    img = Image.new(width, height, [0, 0, 0, 0])

    srand(0)

    # xe[u(16X^[)
    spdtbl = [0, 8, 4, 12, 2, 10, 6, 14, 1, 9, 5, 13, 3, 11, 7, 15]

    # xƑΉ̒
    wtbl = [5, 5, 5, 5, 6, 6, 6, 6, 7, 7, 7, 7, 12, 24, 36, 480]

    # hbg̐Ff[^(3FɌߑł)
    colordata = [[0, 47, 191], [69, 119, 209], [180, 191, 227]]

    # 摜ɑ΂ă_ɐ
    img.height.times do |y|
      # next if y % 16 != 0 and y % 16 != 1
      ymod = y % spdtbl.length

      if len_sync
        # X^[̃XN[xƑΉ̒ɂꍇ
        w = wtbl[spdtbl[ymod]]
      else
        # X^[̃XN[xƑΉĂȂ̒ɂꍇ
        w = wtbl[ymod]
      end

      b = ((spdtbl[ymod] + 1) / spdtbl.length.to_f) * 0.5 + 0.5

      if w == wtbl[-1]
        w = rand(w) + wtbl[0]
        b *= (rand(70) + 31) / 100.0
      end

      l = ((spdtbl.length - ymod) * 0.75).to_i
      l = 1 if l == 0
      l.times do |i|
        x = rand(img.width - w)
        if w <= 8
          wt = [[x, w], [x + 1, w - 2], [x + 2, w - 4]]
        else
          wt = [[x, w], [x + w * 0.08, w * 0.84], [x + w * 0.16, w * 0.68]]
        end
        colordata.each_with_index do |d, i|
          lx, lw = wt[i]
          img.line(lx, y,  lx + lw, y, [d[0] * b, d[1] * b, d[2] *b])
          break if i== 1 and wt[i][0] == wt[i+1][0]
        end
      end
    end

    # 摜ɑ΂ă_ɓ_ł
    num.times do |i|
      c = colordata[rand(3)]
      b = rand(50) / 100.0 + 0.5
      img[rand(img.width), rand(img.height)] = [c[0] * b, c[1] * b, c[2] * b]
    end

    return img
  end

  #
  # mFpBȐ摜쐬ĕԂ
  #
  # @param [int] width   摜(ȗWindow)
  # @param [int] height  摜c(ȗWindowc)
  #
  # @return [Image] 쐬摜(Image)
  #
  def get_vertical_line_image(width=nil, height=nil)
    width = Window.width unless width
    height = Window.height unless height
    img = Image.new(width, height, [0, 0, 0, 0])
    img.line(0, 0, 0, img.height, C_WHITE)
    return img
  end
end

if __FILE__ == $0
  # eXgBg

  spd = 1 # XN[x
  spd_sub = 0.5 # X^[̃XN[x

  # Window.resize(1280, 720)
  # Window.fps = 10
  # "Window.scale = 2.0

  # XN[xnĐ
  shader = StarRasterScrollShader.new(spd, spd_sub, Window.width, Window.height)

  # 摜𐶐E擾(ȂWindow̃TCYō)
  img_nosync = shader.get_star_image(Window.width, Window.height, 0x3fff, false)
  img_sync = shader.get_star_image(Window.width, Window.height, 0x3fff, true)

  # eXgpB摜𐶐E擾
  img_vline = shader.get_vertical_line_image(Window.width, Window.height)

  alpha_chg = false
  framecnt = 0

  # C[v
  Window.loop do
    break if Input.keyPush?(K_ESCAPE)

    # XN[lXV
    shader.update

    # `
    if Input.keyDown?(K_X)
      # XL[Ő\
      Window.draw_shader(0, 0, img_vline, shader)
    elsif Input.keyDown?(K_Z)
      # ZL[ŃXN[xƑΉ摜\
      Window.draw_shader(0, 0, img_sync, shader)
    else
      Window.draw_shader(0, 0, img_nosync, shader)
    end

    # ----------------------------------------
    # ȉ͓eXgp
    #
    # FL[ : tXN[\
    # CL[ : XN[lZbg
    # VL[ : ĂԂ̓XN[xύX
    # AL[ : At@l̕ωؑ
    #
    Window.windowed = !Window.windowed? if Input.keyPush?(K_F)
    shader.reset_scroll if Input.keyPush?(K_C)
    if Input.keyDown?(K_V)
      shader.set_speed(0.5, 0.05)
    else
      shader.set_speed(spd, spd_sub)
    end
    alpha_chg = !alpha_chg if Input.keyPush?(K_A)
    if alpha_chg
      v = Math.sin(framecnt * Math::PI / 180.0).abs
      shader.set_alpha(v)
    else
      shader.set_alpha(1.0)
    end

    framecnt += 1
  end
end
